#-*- coding: utf-8 -*-
# Generated from english.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class EnglishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from english.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_aeo = {u"a", u"e", u"o"}

    g_v = {u"a", u"e", u"i", u"o", u"u", u"y"}

    g_v_WXY = {u"a", u"e", u"i", u"o", u"u", u"y", u"w", u"x", u"Y"}

    g_valid_LI = {u"c", u"d", u"e", u"g", u"h", u"k", u"m", u"n", u"r", u"t"}

    B_Y_found = False
    I_p2 = 0
    I_p1 = 0

    def __r_prelude(self):
        self.B_Y_found = False
        v_1 = self.cursor
        try:
            self.bra = self.cursor
            if not self.eq_s(u"'"):
                raise lab0()
            self.ket = self.cursor
            if not self.slice_del():
                return False

        except lab0: pass
        self.cursor = v_1
        v_2 = self.cursor
        try:
            self.bra = self.cursor
            if not self.eq_s(u"y"):
                raise lab1()
            self.ket = self.cursor
            if not self.slice_from(u"Y"):
                return False
            self.B_Y_found = True
        except lab1: pass
        self.cursor = v_2
        v_3 = self.cursor
        try:
            while True:
                v_4 = self.cursor
                try:
                    try:
                        while True:
                            v_5 = self.cursor
                            try:
                                if not self.in_grouping(EnglishStemmer.g_v):
                                    raise lab5()
                                self.bra = self.cursor
                                if not self.eq_s(u"y"):
                                    raise lab5()
                                self.ket = self.cursor
                                self.cursor = v_5
                                raise lab4()
                            except lab5: pass
                            self.cursor = v_5
                            if self.cursor >= self.limit:
                                raise lab3()
                            self.cursor += 1
                    except lab4: pass
                    if not self.slice_from(u"Y"):
                        return False
                    self.B_Y_found = True
                    continue
                except lab3: pass
                self.cursor = v_4
                break
        except lab2: pass
        self.cursor = v_3
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            try:
                v_2 = self.cursor
                try:
                    if self.find_among(EnglishStemmer.a_0) == 0:
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                if not self.go_out_grouping(EnglishStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                if not self.go_in_grouping(EnglishStemmer.g_v):
                    raise lab0()
                self.cursor += 1
            except lab1: pass
            self.I_p1 = self.cursor
            if not self.go_out_grouping(EnglishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(EnglishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_shortv(self):
        try:
            v_1 = self.limit - self.cursor
            try:
                if not self.out_grouping_b(EnglishStemmer.g_v_WXY):
                    raise lab1()
                if not self.in_grouping_b(EnglishStemmer.g_v):
                    raise lab1()
                if not self.out_grouping_b(EnglishStemmer.g_v):
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            try:
                if not self.out_grouping_b(EnglishStemmer.g_v):
                    raise lab2()
                if not self.in_grouping_b(EnglishStemmer.g_v):
                    raise lab2()
                if self.cursor > self.limit_backward:
                    raise lab2()
                raise lab0()
            except lab2: pass
            self.cursor = self.limit - v_1
            if not self.eq_s_b(u"past"):
                return False
        except lab0: pass
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_Step_1a(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.find_among_b(EnglishStemmer.a_1) == 0:
                self.cursor = self.limit - v_1
                raise lab0()
            self.bra = self.cursor
            if not self.slice_del():
                return False

        except lab0: pass
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_from(u"ss"):
                return False
        elif among_var == 2:
            try:
                v_2 = self.limit - self.cursor
                try:
                    c = self.cursor - 2
                    if c < self.limit_backward:
                        raise lab2()
                    self.cursor = c
                    if not self.slice_from(u"i"):
                        return False
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_2
                if not self.slice_from(u"ie"):
                    return False
            except lab1: pass
        elif among_var == 3:
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            if not self.go_out_grouping_b(EnglishStemmer.g_v):
                return False
            self.cursor -= 1
            if not self.slice_del():
                return False

        return True

    def __r_Step_1b(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_5)
        self.bra = self.cursor
        try:
            v_1 = self.limit - self.cursor
            try:
                if among_var == 1:
                    v_2 = self.limit - self.cursor
                    try:
                        try:
                            v_3 = self.limit - self.cursor
                            try:
                                if self.find_among_b(EnglishStemmer.a_3) == 0:
                                    raise lab4()
                                if self.cursor > self.limit_backward:
                                    raise lab4()
                                raise lab3()
                            except lab4: pass
                            self.cursor = self.limit - v_3
                            if not self.__r_R1():
                                raise lab2()
                            if not self.slice_from(u"ee"):
                                return False
                        except lab3: pass
                    except lab2: pass
                    self.cursor = self.limit - v_2
                elif among_var == 2:
                    raise lab1()
                elif among_var == 3:
                    among_var = self.find_among_b(EnglishStemmer.a_4)
                    if among_var == 0:
                        raise lab1()
                    if among_var == 1:
                        v_4 = self.limit - self.cursor
                        if not self.out_grouping_b(EnglishStemmer.g_v):
                            raise lab1()
                        if self.cursor > self.limit_backward:
                            raise lab1()
                        self.cursor = self.limit - v_4
                        self.bra = self.cursor
                        if not self.slice_from(u"ie"):
                            return False
                    else:
                        if self.cursor > self.limit_backward:
                            raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            v_5 = self.limit - self.cursor
            if not self.go_out_grouping_b(EnglishStemmer.g_v):
                return False
            self.cursor -= 1
            self.cursor = self.limit - v_5
            if not self.slice_del():
                return False

            self.ket = self.cursor
            self.bra = self.cursor
            v_6 = self.limit - self.cursor
            among_var = self.find_among_b(EnglishStemmer.a_6)
            if among_var == 1:
                if not self.slice_from(u"e"):
                    return False
                return False
            elif among_var == 2:
                v_7 = self.limit - self.cursor
                try:
                    if not self.in_grouping_b(EnglishStemmer.g_aeo):
                        raise lab5()
                    if self.cursor > self.limit_backward:
                        raise lab5()
                    return False
                except lab5: pass
                self.cursor = self.limit - v_7
            else:
                if self.cursor != self.I_p1:
                    return False
                v_8 = self.limit - self.cursor
                if not self.__r_shortv():
                    return False
                self.cursor = self.limit - v_8
                if not self.slice_from(u"e"):
                    return False
                return False
            self.cursor = self.limit - v_6
            self.ket = self.cursor
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            self.bra = self.cursor
            if not self.slice_del():
                return False

        except lab0: pass
        return True

    def __r_Step_1c(self):
        self.ket = self.cursor
        try:
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"y"):
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            if not self.eq_s_b(u"Y"):
                return False
        except lab0: pass
        self.bra = self.cursor
        if not self.out_grouping_b(EnglishStemmer.g_v):
            return False
        try:
            if self.cursor > self.limit_backward:
                raise lab2()
            return False
        except lab2: pass
        if not self.slice_from(u"i"):
            return False
        return True

    def __r_Step_2(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_7)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_from(u"tion"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"ence"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"ance"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"able"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"ent"):
                return False
        elif among_var == 6:
            if not self.slice_from(u"ize"):
                return False
        elif among_var == 7:
            if not self.slice_from(u"ate"):
                return False
        elif among_var == 8:
            if not self.slice_from(u"al"):
                return False
        elif among_var == 9:
            if not self.slice_from(u"ful"):
                return False
        elif among_var == 10:
            if not self.slice_from(u"ous"):
                return False
        elif among_var == 11:
            if not self.slice_from(u"ive"):
                return False
        elif among_var == 12:
            if not self.slice_from(u"ble"):
                return False
        elif among_var == 13:
            if not self.slice_from(u"og"):
                return False
        elif among_var == 14:
            if not self.eq_s_b(u"l"):
                return False
            if not self.slice_from(u"og"):
                return False
        elif among_var == 15:
            if not self.slice_from(u"less"):
                return False
        else:
            if not self.in_grouping_b(EnglishStemmer.g_valid_LI):
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Step_3(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_from(u"tion"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"ate"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"al"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"ic"):
                return False
        elif among_var == 5:
            if not self.slice_del():
                return False

        else:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Step_4(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_9)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R2():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        else:
            try:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"s"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                if not self.eq_s_b(u"t"):
                    return False
            except lab0: pass
            if not self.slice_del():
                return False

        return True

    def __r_Step_5(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_10)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            try:
                try:
                    if not self.__r_R2():
                        raise lab1()
                    raise lab0()
                except lab1: pass
                if not self.__r_R1():
                    return False
                v_2 = self.limit - self.cursor
                try:
                    if not self.__r_shortv():
                        raise lab2()
                    return False
                except lab2: pass
                self.cursor = self.limit - v_2
            except lab0: pass
            if not self.slice_del():
                return False

        else:
            if not self.__r_R2():
                return False
            if not self.eq_s_b(u"l"):
                return False
            if not self.slice_del():
                return False

        return True

    def __r_exception1(self):
        self.bra = self.cursor
        among_var = self.find_among(EnglishStemmer.a_11)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if self.cursor < self.limit:
            return False
        if among_var == 1:
            if not self.slice_from(u"sky"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"idl"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"gentl"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"ugli"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"earli"):
                return False
        elif among_var == 6:
            if not self.slice_from(u"onli"):
                return False
        elif among_var == 7:
            if not self.slice_from(u"singl"):
                return False
        return True

    def __r_postlude(self):
        if not self.B_Y_found:
            return False
        while True:
            v_1 = self.cursor
            try:
                try:
                    while True:
                        v_2 = self.cursor
                        try:
                            self.bra = self.cursor
                            if not self.eq_s(u"Y"):
                                raise lab2()
                            self.ket = self.cursor
                            self.cursor = v_2
                            raise lab1()
                        except lab2: pass
                        self.cursor = v_2
                        if self.cursor >= self.limit:
                            raise lab0()
                        self.cursor += 1
                except lab1: pass
                if not self.slice_from(u"y"):
                    return False
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def _stem(self):
        try:
            v_1 = self.cursor
            try:
                if not self.__r_exception1():
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = v_1
            try:
                v_2 = self.cursor
                try:
                    c = self.cursor + 3
                    if c > self.limit:
                        raise lab3()
                    self.cursor = c
                    raise lab2()
                except lab3: pass
                self.cursor = v_2
                raise lab0()
            except lab2: pass
            self.cursor = v_1
            self.__r_prelude()
            self.__r_mark_regions()
            self.limit_backward = self.cursor
            self.cursor = self.limit
            v_5 = self.limit - self.cursor
            self.__r_Step_1a()
            self.cursor = self.limit - v_5
            v_6 = self.limit - self.cursor
            self.__r_Step_1b()
            self.cursor = self.limit - v_6
            v_7 = self.limit - self.cursor
            self.__r_Step_1c()
            self.cursor = self.limit - v_7
            v_8 = self.limit - self.cursor
            self.__r_Step_2()
            self.cursor = self.limit - v_8
            v_9 = self.limit - self.cursor
            self.__r_Step_3()
            self.cursor = self.limit - v_9
            v_10 = self.limit - self.cursor
            self.__r_Step_4()
            self.cursor = self.limit - v_10
            v_11 = self.limit - self.cursor
            self.__r_Step_5()
            self.cursor = self.limit - v_11
            self.cursor = self.limit_backward
            v_12 = self.cursor
            self.__r_postlude()
            self.cursor = v_12
        except lab0: pass
        return True

    a_0 = [
        Among(u"arsen", -1, -1),
        Among(u"commun", -1, -1),
        Among(u"emerg", -1, -1),
        Among(u"gener", -1, -1),
        Among(u"later", -1, -1),
        Among(u"organ", -1, -1),
        Among(u"past", -1, -1),
        Among(u"univers", -1, -1)
    ]

    a_1 = [
        Among(u"'", -1, 1),
        Among(u"'s'", 0, 1),
        Among(u"'s", -1, 1)
    ]

    a_2 = [
        Among(u"ied", -1, 2),
        Among(u"s", -1, 3),
        Among(u"ies", 1, 2),
        Among(u"sses", 1, 1),
        Among(u"ss", 1, -1),
        Among(u"us", 1, -1)
    ]

    a_3 = [
        Among(u"succ", -1, 1),
        Among(u"proc", -1, 1),
        Among(u"exc", -1, 1)
    ]

    a_4 = [
        Among(u"even", -1, 2),
        Among(u"cann", -1, 2),
        Among(u"inn", -1, 2),
        Among(u"earr", -1, 2),
        Among(u"herr", -1, 2),
        Among(u"out", -1, 2),
        Among(u"y", -1, 1)
    ]

    a_5 = [
        Among(u"", -1, -1),
        Among(u"ed", 0, 2),
        Among(u"eed", 1, 1),
        Among(u"ing", 0, 3),
        Among(u"edly", 0, 2),
        Among(u"eedly", 4, 1),
        Among(u"ingly", 0, 2)
    ]

    a_6 = [
        Among(u"", -1, 3),
        Among(u"bb", 0, 2),
        Among(u"dd", 0, 2),
        Among(u"ff", 0, 2),
        Among(u"gg", 0, 2),
        Among(u"bl", 0, 1),
        Among(u"mm", 0, 2),
        Among(u"nn", 0, 2),
        Among(u"pp", 0, 2),
        Among(u"rr", 0, 2),
        Among(u"at", 0, 1),
        Among(u"tt", 0, 2),
        Among(u"iz", 0, 1)
    ]

    a_7 = [
        Among(u"anci", -1, 3),
        Among(u"enci", -1, 2),
        Among(u"ogi", -1, 14),
        Among(u"li", -1, 16),
        Among(u"bli", 3, 12),
        Among(u"abli", 4, 4),
        Among(u"alli", 3, 8),
        Among(u"fulli", 3, 9),
        Among(u"lessli", 3, 15),
        Among(u"ousli", 3, 10),
        Among(u"entli", 3, 5),
        Among(u"aliti", -1, 8),
        Among(u"biliti", -1, 12),
        Among(u"iviti", -1, 11),
        Among(u"tional", -1, 1),
        Among(u"ational", 14, 7),
        Among(u"alism", -1, 8),
        Among(u"ation", -1, 7),
        Among(u"ization", 17, 6),
        Among(u"izer", -1, 6),
        Among(u"ator", -1, 7),
        Among(u"iveness", -1, 11),
        Among(u"fulness", -1, 9),
        Among(u"ousness", -1, 10),
        Among(u"ogist", -1, 13)
    ]

    a_8 = [
        Among(u"icate", -1, 4),
        Among(u"ative", -1, 6),
        Among(u"alize", -1, 3),
        Among(u"iciti", -1, 4),
        Among(u"ical", -1, 4),
        Among(u"tional", -1, 1),
        Among(u"ational", 5, 2),
        Among(u"ful", -1, 5),
        Among(u"ness", -1, 5)
    ]

    a_9 = [
        Among(u"ic", -1, 1),
        Among(u"ance", -1, 1),
        Among(u"ence", -1, 1),
        Among(u"able", -1, 1),
        Among(u"ible", -1, 1),
        Among(u"ate", -1, 1),
        Among(u"ive", -1, 1),
        Among(u"ize", -1, 1),
        Among(u"iti", -1, 1),
        Among(u"al", -1, 1),
        Among(u"ism", -1, 1),
        Among(u"ion", -1, 2),
        Among(u"er", -1, 1),
        Among(u"ous", -1, 1),
        Among(u"ant", -1, 1),
        Among(u"ent", -1, 1),
        Among(u"ment", 15, 1),
        Among(u"ement", 16, 1)
    ]

    a_10 = [
        Among(u"e", -1, 1),
        Among(u"l", -1, 2)
    ]

    a_11 = [
        Among(u"andes", -1, -1),
        Among(u"atlas", -1, -1),
        Among(u"bias", -1, -1),
        Among(u"cosmos", -1, -1),
        Among(u"early", -1, 5),
        Among(u"gently", -1, 3),
        Among(u"howe", -1, -1),
        Among(u"idly", -1, 2),
        Among(u"news", -1, -1),
        Among(u"only", -1, 6),
        Among(u"singly", -1, 7),
        Among(u"skies", -1, 1),
        Among(u"sky", -1, -1),
        Among(u"ugly", -1, 4)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from indonesian.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class IndonesianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from indonesian.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_vowel = {u"a", u"e", u"i", u"o", u"u"}

    I_prefix = 0
    I_measure = 0

    def __r_remove_particle(self):
        self.ket = self.cursor
        if self.find_among_b(IndonesianStemmer.a_0) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        self.I_measure -= 1
        return True

    def __r_remove_possessive_pronoun(self):
        self.ket = self.cursor
        if self.find_among_b(IndonesianStemmer.a_1) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        self.I_measure -= 1
        return True

    def __r_SUFFIX_KAN_OK(self):
        if self.I_prefix == 3:
            return False
        if self.I_prefix == 2:
            return False
        return True

    def __r_SUFFIX_AN_OK(self):
        return self.I_prefix != 1

    def __r_SUFFIX_I_OK(self):
        if self.I_prefix > 2:
            return False
        v_1 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"s"):
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_1
        return True

    def __r_remove_suffix(self):
        self.ket = self.cursor
        if self.find_among_b(IndonesianStemmer.a_2) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        self.I_measure -= 1
        return True

    def __r_VOWEL(self):
        if not self.in_grouping(IndonesianStemmer.g_vowel):
            return False
        return True

    def __r_KER(self):
        if not self.out_grouping(IndonesianStemmer.g_vowel):
            return False
        if not self.eq_s(u"er"):
            return False
        return True

    def __r_remove_first_order_prefix(self):
        self.bra = self.cursor
        among_var = self.find_among(IndonesianStemmer.a_3)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if not self.slice_del():
                return False

            self.I_prefix = 1
            self.I_measure -= 1
        elif among_var == 2:
            if not self.slice_del():
                return False

            self.I_prefix = 3
            self.I_measure -= 1
        elif among_var == 3:
            self.I_prefix = 1
            if not self.slice_from(u"s"):
                return False
            self.I_measure -= 1
        elif among_var == 4:
            self.I_prefix = 3
            if not self.slice_from(u"s"):
                return False
            self.I_measure -= 1
        elif among_var == 5:
            self.I_prefix = 1
            self.I_measure -= 1
            try:
                v_1 = self.cursor
                try:
                    v_2 = self.cursor
                    if not self.in_grouping(IndonesianStemmer.g_vowel):
                        raise lab1()
                    self.cursor = v_2
                    if not self.slice_from(u"p"):
                        return False
                    raise lab0()
                except lab1: pass
                self.cursor = v_1
                if not self.slice_del():
                    return False

            except lab0: pass
        else:
            self.I_prefix = 3
            self.I_measure -= 1
            try:
                v_3 = self.cursor
                try:
                    v_4 = self.cursor
                    if not self.in_grouping(IndonesianStemmer.g_vowel):
                        raise lab3()
                    self.cursor = v_4
                    if not self.slice_from(u"p"):
                        return False
                    raise lab2()
                except lab3: pass
                self.cursor = v_3
                if not self.slice_del():
                    return False

            except lab2: pass
        return True

    def __r_remove_second_order_prefix(self):
        self.bra = self.cursor
        among_var = self.find_among(IndonesianStemmer.a_4)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if not self.slice_del():
                return False

            self.I_prefix = 2
            self.I_measure -= 1
        elif among_var == 2:
            if not self.slice_from(u"ajar"):
                return False
            self.I_measure -= 1
        elif among_var == 3:
            if not self.slice_del():
                return False

            self.I_prefix = 4
            self.I_measure -= 1
        else:
            if not self.slice_from(u"ajar"):
                return False
            self.I_prefix = 4
            self.I_measure -= 1
        return True

    def _stem(self):
        self.I_measure = 0
        v_1 = self.cursor
        try:
            while True:
                try:
                    if not self.go_out_grouping(IndonesianStemmer.g_vowel):
                        raise lab1()
                    self.cursor += 1
                    self.I_measure += 1
                    continue
                except lab1: pass
                break
        except lab0: pass
        self.cursor = v_1
        if self.I_measure <= 2:
            return False
        self.I_prefix = 0
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        self.__r_remove_particle()
        self.cursor = self.limit - v_3
        if self.I_measure <= 2:
            return False
        v_4 = self.limit - self.cursor
        self.__r_remove_possessive_pronoun()
        self.cursor = self.limit - v_4
        self.cursor = self.limit_backward
        if self.I_measure <= 2:
            return False
        try:
            v_5 = self.cursor
            try:
                v_6 = self.cursor
                if not self.__r_remove_first_order_prefix():
                    raise lab3()
                v_7 = self.cursor
                try:
                    v_8 = self.cursor
                    if self.I_measure <= 2:
                        raise lab4()
                    self.limit_backward = self.cursor
                    self.cursor = self.limit
                    if not self.__r_remove_suffix():
                        raise lab4()
                    self.cursor = self.limit_backward
                    self.cursor = v_8
                    if self.I_measure <= 2:
                        raise lab4()
                    if not self.__r_remove_second_order_prefix():
                        raise lab4()
                except lab4: pass
                self.cursor = v_7
                self.cursor = v_6
                raise lab2()
            except lab3: pass
            self.cursor = v_5
            v_9 = self.cursor
            self.__r_remove_second_order_prefix()
            self.cursor = v_9
            v_10 = self.cursor
            try:
                if self.I_measure <= 2:
                    raise lab5()
                self.limit_backward = self.cursor
                self.cursor = self.limit
                if not self.__r_remove_suffix():
                    raise lab5()
                self.cursor = self.limit_backward
            except lab5: pass
            self.cursor = v_10
        except lab2: pass
        return True

    a_0 = [
        Among(u"kah", -1, 1),
        Among(u"lah", -1, 1),
        Among(u"pun", -1, 1)
    ]

    a_1 = [
        Among(u"nya", -1, 1),
        Among(u"ku", -1, 1),
        Among(u"mu", -1, 1)
    ]

    a_2 = [
        Among(u"i", -1, 1, __r_SUFFIX_I_OK),
        Among(u"an", -1, 1, __r_SUFFIX_AN_OK),
        Among(u"kan", 1, 1, __r_SUFFIX_KAN_OK)
    ]

    a_3 = [
        Among(u"di", -1, 1),
        Among(u"ke", -1, 2),
        Among(u"me", -1, 1),
        Among(u"mem", 2, 5),
        Among(u"men", 2, 1),
        Among(u"meng", 4, 1),
        Among(u"meny", 4, 3, __r_VOWEL),
        Among(u"pem", -1, 6),
        Among(u"pen", -1, 2),
        Among(u"peng", 8, 2),
        Among(u"peny", 8, 4, __r_VOWEL),
        Among(u"ter", -1, 1)
    ]

    a_4 = [
        Among(u"be", -1, 3, __r_KER),
        Among(u"belajar", 0, 4),
        Among(u"ber", 0, 3),
        Among(u"pe", -1, 1),
        Among(u"pelajar", 3, 2),
        Among(u"per", 3, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from romanian.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class RomanianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from romanian.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_v = {u"a", u"e", u"i", u"o", u"u", u"â", u"î", u"ă"}

    B_standard_suffix_removed = False
    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_norm(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    try:
                        while True:
                            v_3 = self.cursor
                            try:
                                self.bra = self.cursor
                                among_var = self.find_among(RomanianStemmer.a_0)
                                if among_var == 0:
                                    raise lab3()
                                self.ket = self.cursor
                                if among_var == 1:
                                    if not self.slice_from(u"ș"):
                                        return False
                                else:
                                    if not self.slice_from(u"ț"):
                                        return False
                                self.cursor = v_3
                                raise lab2()
                            except lab3: pass
                            self.cursor = v_3
                            if self.cursor >= self.limit:
                                raise lab1()
                            self.cursor += 1
                    except lab2: pass
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_prelude(self):
        while True:
            v_1 = self.cursor
            try:
                try:
                    while True:
                        v_2 = self.cursor
                        try:
                            if not self.in_grouping(RomanianStemmer.g_v):
                                raise lab2()
                            self.bra = self.cursor
                            try:
                                v_3 = self.cursor
                                try:
                                    if not self.eq_s(u"u"):
                                        raise lab4()
                                    self.ket = self.cursor
                                    if not self.in_grouping(RomanianStemmer.g_v):
                                        raise lab4()
                                    if not self.slice_from(u"U"):
                                        return False
                                    raise lab3()
                                except lab4: pass
                                self.cursor = v_3
                                if not self.eq_s(u"i"):
                                    raise lab2()
                                self.ket = self.cursor
                                if not self.in_grouping(RomanianStemmer.g_v):
                                    raise lab2()
                                if not self.slice_from(u"I"):
                                    return False
                            except lab3: pass
                            self.cursor = v_2
                            raise lab1()
                        except lab2: pass
                        self.cursor = v_2
                        if self.cursor >= self.limit:
                            raise lab0()
                        self.cursor += 1
                except lab1: pass
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            try:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(RomanianStemmer.g_v):
                        raise lab2()
                    try:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(RomanianStemmer.g_v):
                                raise lab4()
                            if not self.go_out_grouping(RomanianStemmer.g_v):
                                raise lab4()
                            self.cursor += 1
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_3
                        if not self.in_grouping(RomanianStemmer.g_v):
                            raise lab2()
                        if not self.go_in_grouping(RomanianStemmer.g_v):
                            raise lab2()
                        self.cursor += 1
                    except lab3: pass
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                if not self.out_grouping(RomanianStemmer.g_v):
                    raise lab0()
                try:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(RomanianStemmer.g_v):
                            raise lab6()
                        if not self.go_out_grouping(RomanianStemmer.g_v):
                            raise lab6()
                        self.cursor += 1
                        raise lab5()
                    except lab6: pass
                    self.cursor = v_4
                    if not self.in_grouping(RomanianStemmer.g_v):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                except lab5: pass
            except lab1: pass
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(RomanianStemmer.g_v):
                raise lab7()
            self.cursor += 1
            if not self.go_in_grouping(RomanianStemmer.g_v):
                raise lab7()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(RomanianStemmer.g_v):
                raise lab7()
            self.cursor += 1
            if not self.go_in_grouping(RomanianStemmer.g_v):
                raise lab7()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab7: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(RomanianStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"i"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"u"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_step_0(self):
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"a"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"e"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"i"):
                return False
        elif among_var == 5:
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"ab"):
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.slice_from(u"i"):
                return False
        elif among_var == 6:
            if not self.slice_from(u"at"):
                return False
        else:
            if not self.slice_from(u"ați"):
                return False
        return True

    def __r_combo_suffix(self):
        v_1 = self.limit - self.cursor
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            if not self.slice_from(u"abil"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"ibil"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"iv"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"ic"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"at"):
                return False
        else:
            if not self.slice_from(u"it"):
                return False
        self.B_standard_suffix_removed = True
        self.cursor = self.limit - v_1
        return True

    def __r_standard_suffix(self):
        self.B_standard_suffix_removed = False
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.__r_combo_suffix():
                    raise lab0()
                continue
            except lab0: pass
            self.cursor = self.limit - v_1
            break
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R2():
            return False
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.eq_s_b(u"ț"):
                return False
            self.bra = self.cursor
            if not self.slice_from(u"t"):
                return False
        else:
            if not self.slice_from(u"ist"):
                return False
        self.B_standard_suffix_removed = True
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_5)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        if among_var == 1:
            try:
                v_3 = self.limit - self.cursor
                try:
                    if not self.out_grouping_b(RomanianStemmer.g_v):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_3
                if not self.eq_s_b(u"u"):
                    self.limit_backward = v_2
                    return False
            except lab0: pass
            if not self.slice_del():
                return False

        else:
            if not self.slice_del():
                return False

        self.limit_backward = v_2
        return True

    def __r_vowel_suffix(self):
        self.ket = self.cursor
        if self.find_among_b(RomanianStemmer.a_6) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_RV():
            return False
        if not self.slice_del():
            return False

        return True

    def _stem(self):
        self.__r_norm()
        v_2 = self.cursor
        self.__r_prelude()
        self.cursor = v_2
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_4 = self.limit - self.cursor
        self.__r_step_0()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_standard_suffix()
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        try:
            try:
                v_7 = self.limit - self.cursor
                try:
                    if not self.B_standard_suffix_removed:
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_7
                if not self.__r_verb_suffix():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_6
        v_8 = self.limit - self.cursor
        self.__r_vowel_suffix()
        self.cursor = self.limit - v_8
        self.cursor = self.limit_backward
        v_9 = self.cursor
        self.__r_postlude()
        self.cursor = v_9
        return True

    a_0 = [
        Among(u"ş", -1, 1),
        Among(u"ţ", -1, 2)
    ]

    a_1 = [
        Among(u"", -1, 3),
        Among(u"I", 0, 1),
        Among(u"U", 0, 2)
    ]

    a_2 = [
        Among(u"ea", -1, 3),
        Among(u"ația", -1, 7),
        Among(u"aua", -1, 2),
        Among(u"iua", -1, 4),
        Among(u"ație", -1, 7),
        Among(u"ele", -1, 3),
        Among(u"ile", -1, 5),
        Among(u"iile", 6, 4),
        Among(u"iei", -1, 4),
        Among(u"atei", -1, 6),
        Among(u"ii", -1, 4),
        Among(u"ului", -1, 1),
        Among(u"ul", -1, 1),
        Among(u"elor", -1, 3),
        Among(u"ilor", -1, 4),
        Among(u"iilor", 14, 4)
    ]

    a_3 = [
        Among(u"icala", -1, 4),
        Among(u"iciva", -1, 4),
        Among(u"ativa", -1, 5),
        Among(u"itiva", -1, 6),
        Among(u"icale", -1, 4),
        Among(u"ațiune", -1, 5),
        Among(u"ițiune", -1, 6),
        Among(u"atoare", -1, 5),
        Among(u"itoare", -1, 6),
        Among(u"ătoare", -1, 5),
        Among(u"icitate", -1, 4),
        Among(u"abilitate", -1, 1),
        Among(u"ibilitate", -1, 2),
        Among(u"ivitate", -1, 3),
        Among(u"icive", -1, 4),
        Among(u"ative", -1, 5),
        Among(u"itive", -1, 6),
        Among(u"icali", -1, 4),
        Among(u"atori", -1, 5),
        Among(u"icatori", 18, 4),
        Among(u"itori", -1, 6),
        Among(u"ători", -1, 5),
        Among(u"icitati", -1, 4),
        Among(u"abilitati", -1, 1),
        Among(u"ivitati", -1, 3),
        Among(u"icivi", -1, 4),
        Among(u"ativi", -1, 5),
        Among(u"itivi", -1, 6),
        Among(u"icităi", -1, 4),
        Among(u"abilităi", -1, 1),
        Among(u"ivităi", -1, 3),
        Among(u"icități", -1, 4),
        Among(u"abilități", -1, 1),
        Among(u"ivități", -1, 3),
        Among(u"ical", -1, 4),
        Among(u"ator", -1, 5),
        Among(u"icator", 35, 4),
        Among(u"itor", -1, 6),
        Among(u"ător", -1, 5),
        Among(u"iciv", -1, 4),
        Among(u"ativ", -1, 5),
        Among(u"itiv", -1, 6),
        Among(u"icală", -1, 4),
        Among(u"icivă", -1, 4),
        Among(u"ativă", -1, 5),
        Among(u"itivă", -1, 6)
    ]

    a_4 = [
        Among(u"ica", -1, 1),
        Among(u"abila", -1, 1),
        Among(u"ibila", -1, 1),
        Among(u"oasa", -1, 1),
        Among(u"ata", -1, 1),
        Among(u"ita", -1, 1),
        Among(u"anta", -1, 1),
        Among(u"ista", -1, 3),
        Among(u"uta", -1, 1),
        Among(u"iva", -1, 1),
        Among(u"ic", -1, 1),
        Among(u"ice", -1, 1),
        Among(u"abile", -1, 1),
        Among(u"ibile", -1, 1),
        Among(u"isme", -1, 3),
        Among(u"iune", -1, 2),
        Among(u"oase", -1, 1),
        Among(u"ate", -1, 1),
        Among(u"itate", 17, 1),
        Among(u"ite", -1, 1),
        Among(u"ante", -1, 1),
        Among(u"iste", -1, 3),
        Among(u"ute", -1, 1),
        Among(u"ive", -1, 1),
        Among(u"ici", -1, 1),
        Among(u"abili", -1, 1),
        Among(u"ibili", -1, 1),
        Among(u"iuni", -1, 2),
        Among(u"atori", -1, 1),
        Among(u"osi", -1, 1),
        Among(u"ati", -1, 1),
        Among(u"itati", 30, 1),
        Among(u"iti", -1, 1),
        Among(u"anti", -1, 1),
        Among(u"isti", -1, 3),
        Among(u"uti", -1, 1),
        Among(u"iști", -1, 3),
        Among(u"ivi", -1, 1),
        Among(u"ităi", -1, 1),
        Among(u"oși", -1, 1),
        Among(u"ități", -1, 1),
        Among(u"abil", -1, 1),
        Among(u"ibil", -1, 1),
        Among(u"ism", -1, 3),
        Among(u"ator", -1, 1),
        Among(u"os", -1, 1),
        Among(u"at", -1, 1),
        Among(u"it", -1, 1),
        Among(u"ant", -1, 1),
        Among(u"ist", -1, 3),
        Among(u"ut", -1, 1),
        Among(u"iv", -1, 1),
        Among(u"ică", -1, 1),
        Among(u"abilă", -1, 1),
        Among(u"ibilă", -1, 1),
        Among(u"oasă", -1, 1),
        Among(u"ată", -1, 1),
        Among(u"ită", -1, 1),
        Among(u"antă", -1, 1),
        Among(u"istă", -1, 3),
        Among(u"ută", -1, 1),
        Among(u"ivă", -1, 1)
    ]

    a_5 = [
        Among(u"ea", -1, 1),
        Among(u"ia", -1, 1),
        Among(u"esc", -1, 1),
        Among(u"ăsc", -1, 1),
        Among(u"ind", -1, 1),
        Among(u"ând", -1, 1),
        Among(u"are", -1, 1),
        Among(u"ere", -1, 1),
        Among(u"ire", -1, 1),
        Among(u"âre", -1, 1),
        Among(u"se", -1, 2),
        Among(u"ase", 10, 1),
        Among(u"sese", 10, 2),
        Among(u"ise", 10, 1),
        Among(u"use", 10, 1),
        Among(u"âse", 10, 1),
        Among(u"ește", -1, 1),
        Among(u"ăște", -1, 1),
        Among(u"eze", -1, 1),
        Among(u"ai", -1, 1),
        Among(u"eai", 19, 1),
        Among(u"iai", 19, 1),
        Among(u"sei", -1, 2),
        Among(u"ești", -1, 1),
        Among(u"ăști", -1, 1),
        Among(u"ui", -1, 1),
        Among(u"ezi", -1, 1),
        Among(u"âi", -1, 1),
        Among(u"ași", -1, 1),
        Among(u"seși", -1, 2),
        Among(u"aseși", 29, 1),
        Among(u"seseși", 29, 2),
        Among(u"iseși", 29, 1),
        Among(u"useși", 29, 1),
        Among(u"âseși", 29, 1),
        Among(u"iși", -1, 1),
        Among(u"uși", -1, 1),
        Among(u"âși", -1, 1),
        Among(u"ați", -1, 2),
        Among(u"eați", 38, 1),
        Among(u"iați", 38, 1),
        Among(u"eți", -1, 2),
        Among(u"iți", -1, 2),
        Among(u"âți", -1, 2),
        Among(u"arăți", -1, 1),
        Among(u"serăți", -1, 2),
        Among(u"aserăți", 45, 1),
        Among(u"seserăți", 45, 2),
        Among(u"iserăți", 45, 1),
        Among(u"userăți", 45, 1),
        Among(u"âserăți", 45, 1),
        Among(u"irăți", -1, 1),
        Among(u"urăți", -1, 1),
        Among(u"ârăți", -1, 1),
        Among(u"am", -1, 1),
        Among(u"eam", 54, 1),
        Among(u"iam", 54, 1),
        Among(u"em", -1, 2),
        Among(u"asem", 57, 1),
        Among(u"sesem", 57, 2),
        Among(u"isem", 57, 1),
        Among(u"usem", 57, 1),
        Among(u"âsem", 57, 1),
        Among(u"im", -1, 2),
        Among(u"âm", -1, 2),
        Among(u"ăm", -1, 2),
        Among(u"arăm", 65, 1),
        Among(u"serăm", 65, 2),
        Among(u"aserăm", 67, 1),
        Among(u"seserăm", 67, 2),
        Among(u"iserăm", 67, 1),
        Among(u"userăm", 67, 1),
        Among(u"âserăm", 67, 1),
        Among(u"irăm", 65, 1),
        Among(u"urăm", 65, 1),
        Among(u"ârăm", 65, 1),
        Among(u"au", -1, 1),
        Among(u"eau", 76, 1),
        Among(u"iau", 76, 1),
        Among(u"indu", -1, 1),
        Among(u"ându", -1, 1),
        Among(u"ez", -1, 1),
        Among(u"ească", -1, 1),
        Among(u"ară", -1, 1),
        Among(u"seră", -1, 2),
        Among(u"aseră", 84, 1),
        Among(u"seseră", 84, 2),
        Among(u"iseră", 84, 1),
        Among(u"useră", 84, 1),
        Among(u"âseră", 84, 1),
        Among(u"iră", -1, 1),
        Among(u"ură", -1, 1),
        Among(u"âră", -1, 1),
        Among(u"ează", -1, 1)
    ]

    a_6 = [
        Among(u"a", -1, 1),
        Among(u"e", -1, 1),
        Among(u"ie", 1, 1),
        Among(u"i", -1, 1),
        Among(u"ă", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass
